

	device	PIC16F870,pwrte_on,wdt_off,hs_osc

VERS	=	035

LPTIME	=	42990		;calibration time for counter

MAXMODE	=	17		;total operational modes

CLAMP	=	ra.5		;clamp input to T0
R20	=	ra.0		;20 ohm direct feed
R100	=	ra.1		;100 ohms
R470	=	ra.4		;470 ohms
R150	=	ra.2		;150
R10K	=	rc.7		;10k
R10KA	=	rc.6		;10k
R100K	=	ra.3		;100k feed to input

BLACK	=	R150
WHITE	=	R100
SOUT	=	R20

BUT1	=	rc.4		;button 1
BUT2	=	rc.5		;button 2


	org	20h

wsave	ds	1
ssave	ds	1
icount	ds	1

flag	ds	1
count	ds	1
scount	ds	1
csave	ds	1
data	ds	1
sdata	ds	1
temp	ds	2
ontime	ds	1
oftime	ds	1
isrvec	ds	1
qtime	ds	2		;comm timer for serial
timer	ds	3
freq	ds	2		;
pwmp	ds	1
mode	ds	1
rate	ds	1
baud	ds	1
midic	ds	1		;midi channel
pmode	ds	1
hi	ds	1
low	ds	1
rand	ds	3
acc	ds	4		;32 bit register
xacc	ds	4		;aux 32 bit value
digits	ds	4		;segment data for LED
dignr	ds	1
dp	ds	1
ftimer	ds	2
segmask	ds	1
dela	ds	3
bcd	ds	5		;10 digit bcd buffer
pto	ds	1
pti	ds	1
cnt	ds	1
ii	ds	1
frame	ds	1


HOLD	=	flag.0
DIR	=	flag.1
SAVE	=	flag.2
RUN	=	flag.3
SWIT	=	flag.4


	org	0

	goto	start

	org	4
	
isr	movwf	wsave
	swapf	status,w
	movwf	ssave
	clrf	status
	
	movf	isrvec,w
	addwf	PCL
	goto	pwmi
	goto	sqri
		
isrx	swapf	ssave,w
	movwf	status
	swapf	wsave
	swapf	wsave,w
	reti

digseg	addwf	PCL
	retw	3fh,6h,5bh,4fh,66h,6dh,7dh,7h,7fh,67h

getano	addwf	PCL
	retw	1,2,4,8


;	bit 7	decimal point
;	bit 6	flash decimal
;	bit 5,4	dp location
;	bit 3-0	starting digit from bcd buffer
	
getdp	addwf	PCL
	retw	03h,03h,03h,03h
	retw	94h,0A5h,0C6h,0D7h
	retw	0E8h,0F9h

dpcap	addwf	PCL
	retw	01h,01h,84h,84h
	retw	84h,95h,0A6h,0B7
	retw	0C8h,0D9h


dpuls	addwf	PCL
	retw	170,55,17,5

srq	movf	temp,w
	incf	temp
	addwf	PCL
	retw	1380>,1380<		;1200
	retw	680>,680<		;2400
	retw	330>,330<		;4800
	retw	160>,160<		;9600

modes	movf	temp,w
	incf	temp
	addwf	PCL
vbase	retw	73h,50h,5ch,7ch		;Prob
	retw	73h,3eh,38h,6dh		;PULS
	retw	3eh,5ch,38h,78h		;VoLt
	retw	05eh,4,5ch,05eh		;diod
	retw	71h,50h,79h,6fh		;FrEq
	retw	0,39h,54h,78h		; Cnt
	retw	0,39h,77h,73h		; CAP
	retw	39h,5ch,4,38h		;CoiL
	retw	0,6dh,6,3dh		; SIG
	retw	54h,78h,6dh,39h		;ntSC
	retw	00h,6dh,79h,50h		; SEr
	retw	15h,4,5eh,4		;Midi
	retw	21h,52h,58h,0		;r/c
	retw	39h,0fh,00h,0		;[]
	retw	73h,50h,54h,0		;Prn
	retw	04h,50h,4fh,7fh		;ir38
	retw	73h,2ah,15h,0		;PWm

vpuls	retw	0,0,0,6dh		;   5
	retw	0,0,6dh,3fh		;  50
	retw	0,6dh,3fh,3fh		; 500
	retw	0,0,0edh,3fh		; 5.0

vbaud	retw	06h,5bh,3fh,3fh		;1200	
	retw	5bh,66h,3fh,3fh		;2400
	retw	66h,7fh,3fh,3fh		;4800
	retw	67h,7dh,3fh,3fh		;9600

; execute routine as selected from menu

exec	movf	mode,w
	addwf	PCL
	goto	lp1		;logic probe
	goto	pulse		;logic pulser
	goto	volt		;voltmeter
	goto	diode		;doide voltage
	goto	fcount		;frequency count
	goto	dcount		;event count
	goto	cap		;cap meter
	goto	induct		;inducance meter
	goto	sig		;signal generator
	goto	vgen		;video generator
	goto	s9600		;serial out
	goto	midi		;midi out
	goto	radcon		;servo pulse
	goto	square		;square wave
	goto	noise		;digital noise
	goto	ir38		;38 khz test signal
	goto	pwm		;variable pulse width
		



start	bsf	status,5
	movlw	111111b	
	movwf	TRISA
	movlw	0
	movwf	TRISB
	movlw	10110000b
	movwf	TRISC
	movlw	10001000b
	movwf	OPTION
	bcf	status,5
	movlw	4
	movwf	T2CON
	clrf	segmask
	bsf	segmask,0
	clrf	dignr
	call	getset		;restore current mode
	call	signon
	movlw	1000>
	movwf	freq
	movlw	1000<
	movwf	freq+1
	movlw	48
	movwf	pwmp

; menu selection of operational modes

cycmode	call	putset		;save all parameters
 
cycle	bcf	GIE
	movf	mode,w
	call	setmode		;load segment menu data
	
:c2	call	segout		;run display
	movlw	10
	call	delay
	btfsc	BUT1		;still held down?
	goto	:c4		;no
	btfss	BUT2
	goto	:c2

:c5	call	segout
	movlw	10
	call	delay
	btfsc	BUT1
	goto	doexec
	btfsc	BUT2
	goto	:c5
	decf	mode
	movlw	MAXMODE-1
	btfsc	mode,7		;underflow?
	movwf	mode
	goto	cycle

:c4	call	segout
	movlw	10
	call	delay
	btfsc	BUT2
	goto	doexec		;both buttons up - execute
	btfsc	BUT1
	goto	:c4
	incf	mode
	movf	mode,w
	xorlw	MAXMODE
	btfsc	z
	clrf	mode
	goto	cycle

doexec	call	putset		;first, save op mode
	call	clreg
	goto	exec

clreg	bsf	RP0		;then clear all lines
	bsf	R20
	bsf	R100
	bsf	R470
	bsf	R150
	bsf	R10K
	bsf	R10KA
	bsf	R100K
	bsf	CLAMP
	movlw	6
	movwf	ADCON1	
	bcf	RP0
	ret

; save current setup in eeprom


putopt	btfss	SAVE		;flagged for save?
	ret			;no.
	bcf	SAVE		;yes.
putset	clrf	hi
	clrf	low
	movf	mode,w
	call	eewrite		;save current mode in eeprom
	incf	low
	movf	pmode,w
	call	eewrite
	incf	low
	movf	midic,w
	call	eewrite
	incf	low
	movf	baud,w
	call	eewrite
	ret


; get op mode from eeprom

getset	clrf	hi
	clrf	low
	call	eeread
	movwf	mode
	movf	mode,w
	addlw	-MAXMODE
	btfsc	c
	clrf	mode		;restore
	incf	low		;next location
	call	eeread
	movwf	pmode		;pulse mode
	movf	pmode,w
	addlw	-4
	btfsc	c
	clrf	pmode
	incf	low
	call	eeread
	movwf	midic
	addlw	-15
	btfsc	c
	clrf	midic
	incf	low		;next location
	call	eeread
	movwf	baud		;pulse mode
	movf	baud,w
	addlw	-4
	btfsc	c
	clrf	baud
	ret


; put mode text on 4 char display

setmode	movwf	temp
	bcf	c
	rlf	temp
	rlf	temp
	call	modes
	movwf	digits
	call	modes
	movwf	digits+1
	call	modes
	movwf	digits+2
	call	modes
	movwf	digits+3
	ret


; show version nr and op mode at startup

signon	movlw	VERS
	call	wtod		;put version nr
	movlw	0		;blank.
	movwf	digits
	call	sign
	movf	mode,w
	call	setmode
	call	sign
	ret

; run display slow for flashing effect

sign	clrf	count
:x1	call	segout
	movlw	15
	call	delay
	decfsz	count
	goto	:x1
	ret

; PWM interrupt handler

pwmi	bcf	T0IF
	btfss	SWIT
	goto	:on
	bcf	SWIT
:off	bcf	R20
	movf	oftime,w
	movwf	TMR0
	goto	isrx
:on	bsf	SWIT
	bsf	R20
	movf	ontime,w
	movwf	TMR0
	goto	isrx

; PWM control routine

pwm	bsf	RP0
	bcf	R20
	movlw	10000010b	;/32 TMR0
	movwf	OPTION		
	bcf	RP0
	
	movlw	0
	movwf	isrvec		;set interrupt re-vector
	bsf	T0IE
	bsf	GIE
	
:2	movf	pwmp,w
	addlw	2
	call	wtod
	movf	pwmp,w
	sublw	0
	movwf	ontime
	movf	pwmp,w
	sublw	96
	sublw	0
	movwf	oftime
:4	movlw	10
	call	delay
	call	segout
	movlw	10
	call	delay
	call	segout
	decfsz	timer
	goto	:4
	movlw	25
	movwf	timer
:3	movlw	5
	call	delay
	call	segout
	btfss	BUT1
	goto	:dn
	btfss	BUT2
	goto	:up
	clrf	timer
	goto	:3


:dn	btfss	BUT2
	goto	cycle
	decfsz	pwmp
	decf	pwmp
	incf	pwmp
	goto	:2
	
:up	btfss	BUT1
	goto	cycle
	incf	pwmp
	movf	pwmp,w
	xorlw	96
	btfsc	z
	decf	pwmp
	goto	:2



ir38	bsf	RP0
	bcf	R20
	bcf	RP0
	
:2	call	:brst
	movlw	5
	movwf	count
:4	call	segout
	movlw	9
	call	delay
	decfsz	count
	goto	:4
	call	segoff
	btfsc	BUT1
	goto	:2
	btfsc	BUT2
	goto	:2
	goto	cycle

:brst	movlw	38
	movwf	count
:b2	bsf	R20
	movlw	20
	movwf	dela
:b3	decfsz	dela
	goto	:b3
	nop
	nop
	nop
	bcf	R20
	movlw	20
	movwf	dela
:b5	decfsz	dela
	goto	:b5
	nop
	decfsz	count
	goto	:b2
	ret
	



; 20 khz digital noise @20Mhz

noise	bsf	RP0
	bcf	R20
	bcf	RP0
	movlw	55h
	movwf	rand
:2	call	pseudo
	btfsc	rand,0
	bsf	R20
	btfss	rand,0
	bcf	R20
	call	segout
	movlw	65
	movwf	dela
:4	decfsz	dela
	goto	:4
	btfsc	BUT2
	goto	:2
	btfsc	BUT1
	goto	:2
	goto	cycle


pseudo	movf	rand,w
	addwf	rand+1,w
	movwf	rand+1
	addwf	rand+2,w
	movwf	rand+2
	bcf	c		;do 8 bit rotate
	rrf	rand
	btfsc	c
	bsf	rand,7
	addwf	rand
	ret


; pulse width generation @20Mhz

radcon	bsf	RP0
	bcf	R20
	bcf	RP0
	movlw	1500>
	movwf	timer
	movlw	1500<
	movwf	timer+1
:1	movf	timer,w
	movwf	acc
	movf	timer+1,w
	movwf	acc+1
	clrf	acc+2
	clrf	acc+3
	call	b2bcd
	call	format
:2	movlw	50
	movwf	count
:3	call	segout
	movlw	10
	call	delay
	decfsz	count
	goto	:3
	movf	timer,w
	movwf	low
	movf	timer+1,w
	movwf	hi
	call	segoff
	call	:pls
	btfss	BUT1
	goto	:dn
	btfss	BUT2
	goto	:up
	goto	:2
	
:up	btfss	BUT1
	goto	cycle
:up2	movlw	10
	addwf	timer
	btfsc	c
	incf	timer+1
	movf	timer+1,w
	xorlw	9
	btfsc	z
	goto	:dn2
	goto	:1

:dn	btfss	BUT2
	goto	cycle
:dn2	movlw	-10
	addwf	timer
	btfss	c
	decf	timer+1
	movf	timer+1,w
	xorlw	2
	btfsc	z
	goto	:up2
	goto	:1
	
:pls	incf	hi
	incf	low
	bsf	R20
:4	nop
	nop
:5	decfsz	low
	goto	:4
	decfsz	hi
	goto	:5
	bcf	R20
	ret


; Square wave interrupt handler

sqri	bcf	T0IF
	movf	freq,w
	addwf	timer
	movf	freq+1,w
	btfsc	c
	incfsz	freq+1,w
	addwf	timer+1
	btfss	timer+1,7
	bcf	R20
	btfsc	timer+1,7
	bsf	R20
	movlw	-73
	addwf	TMR0
	goto	isrx


; linear squarewave generation @20Mhz

square	bsf	RP0
	bcf	R20
	movlw	10001000b
	movwf	OPTION		;/32 clock
	bcf	RP0
	movlw	1
	movwf	isrvec
	bsf	GIE
	bsf	T0IE		;timer-0 interrupt on

:hl	movf	freq,w
	movwf	acc
	movf	freq+1,w
	movwf	acc+1
	clrf	acc+2
	clrf	acc+3
	call	b2bcd
	call	format

:2	movf	rate,w
	movwf	count
:s1	call	segout
	movlw	10
	call	delay
	decfsz	count
	goto	:s1
	movlw	-5
	addwf	rate
	movf	rate,w
	addlw	-5
	btfsc	c
	goto	:4
	movlw	6
	movwf	rate
:4	btfss	BUT1
	goto	:dn
	btfss	BUT2
	goto	:up
	clrf	rate
	goto	:2

:up	btfss	BUT1
	goto	cycle
	incf	freq
	btfsc	z
	incf	freq+1
	goto	:hl

:dn	btfss	BUT2
	goto	cycle
	movlw	-1
	addwf	freq
	btfss	c
	decf	freq+1
	btfss	freq+1,7
	goto	:hl
	clrf	freq
	clrf	freq+1
	goto	:hl


; logic pulser

pulse	bsf	RP0
	movlw	6
	movwf	ADCON1		;all digital on ra
	bcf	RP0

:p2	movf	pmode,w		;get pulse mode
	addlw	vpuls-vbase/4	;NOTE: = (vpuls-vbase)/4 in this assember
	call	setmode
	movf	pmode,w		;now, get delay factor
	call	dpuls
	movwf	timer
	goto	:p3
	
:p5	bsf	RP0
	bcf	R20		;activate pulse
	nop
	bsf	R20		;end of pulse
	bcf	RP0
	bsf	digits,6	;show pulse is active
	goto	:p8
	
:p3	bsf	R20		;set latch to low.
	btfsc	R470		;which way to pulse?
	bcf	R20		;then set latch to high.
	movlw	8		;low level detected?
	btfsc	R20
	movlw	1		;no - show high
	movwf	digits		;in 1st display
:p8	movf	timer,w
	call	delsq		;delay w/display
	movlw	2
	call	delsq		;adjust for hi end error
	btfsc	BUT1
	goto	:p4		;no button 1.
	btfsc	BUT2
	goto	:p5		;but 1 only - do pulses
	goto	cycmode		;exit to menu.

:p4	btfsc	BUT2		;only button 2 ?
	goto	:p3		;no - set pulse direction
:p4x	movlw	10
	call	delay
	call	blank
	call	segout
	btfss	BUT1		;exit?
	goto	cycmode		;yes - and save params
	btfss	BUT2
	goto	:p4x
	incf	pmode
	movf	pmode,w
	sublw	3
	btfss	c
	clrf	pmode
	goto	:p2


; signal generator (0.5 v square wave)

sig	bsf	RP0
	bcf	R10K		;engage 10k resistor
	bcf	R100K		;engage 100k resistor
	movlw	6
	movwf	ADCON1		;all digital on ra
	bcf	RP0
	bcf	R10K		;10k pull to ground
	bcf	R100K		;100k for signal gen
	clrf	digits
	movlw	5
	call	digseg
	movwf	digits+1
	movlw	0
	call	digseg
	movwf	digits+2
	movwf	digits+3
		
	
:s3	btfss	timer,6
	bsf	digits,6
	bsf	R100K
	call	:s8
:s4	incf	timer
	btfsc	timer,6
	bcf	digits,6
	bcf	R100K
	call	:s8
	btfsc	BUT1
	goto	:s4
	btfsc	BUT2
	goto	:s3
	goto	cycle

:s8	movlw	32
	movwf	count
:s9	call	segout		;run diplay
	movlw	14
	call	short
	decfsz	count
	goto	:s9
	ret


; do short delays

short	movwf	dela
:s2	nop
	nop
	nop
	nop
	nop
	decfsz	dela
	goto	:s2
	ret

	

; measure capacitor from tip to gound lead

cap	bsf	RP0
	bcf	R100K		;enable 100k resistor
	bcf	RP0
	bsf	R100K		;pull up resitor
	bcf	R20		;pull down for clamp
	bsf	HOLD		;default hold condition

:c2	call	segoff
	clrf	acc		;clear accum
	clrf	acc+1
	clrf	acc+2
	clrf	acc+3
	bsf	RP0
	bsf	R20		;release clamp
	bcf	RP0
	nop
	nop			;compensate
	nop			;for stray
	nop			;capacitance
	nop
	nop
			
:c7	nop			;tune loop with nop's
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	btfsc	R20		;check direct (a/d) input
	goto	:hit
	incfsz	acc
	goto	:c7
	incfsz	acc+1
	goto	:c7
	incfsz	acc+2
	goto	:c7
	incfsz	acc+3
	goto	:c7

:hit	bsf	RP0
	bcf	R20		;discharge cap
	bcf	RP0
	call	b2bcd
	call	cform		;format for caps
:c6	call	show2
	btfsc	BUT1
	goto	:c6		;no button - hold reading
	btfsc	BUT2
	goto	:c2		;but1 only - do reading
	goto	cycle


; measure inductor from tip to gound lead

induct	bsf	RP0
	bcf	R100		;enable 470 ohm resistor
	bcf	RP0
	bcf	R100		;pull up resitor
	bsf	HOLD		;default hold condition

:c2	call	segoff
	movlw	80
	call	delay
	clrf	acc		;clear accum
	clrf	acc+1
	clrf	acc+2
	clrf	acc+3
	bsf	R100		;start voltage
	
:c7	btfss	R20		;check direct (a/d) input
	goto	:hit
	incfsz	acc
	goto	:c7
	incfsz	acc+1
	goto	:c7
	incfsz	acc+2
	goto	:c7
	incfsz	acc+3
	goto	:c7

:hit	bcf	R100
	call	b2bcd
	movlw	3
	call	xform		
	bsf	digits+2,7
:c6	call	show2
	btfsc	BUT1
	goto	:c6		;no button - hold reading
	btfsc	BUT2
	goto	:c2		;but1 only - do reading
	goto	cycle


; turn off all digits

segoff	bcf	rc,0
	bcf	rc,1
	bcf	rc,2
	bcf	rc,3
	ret


; Logic probe function
; show Low, High, and Float w/ Pulse detection

lp1	bsf	RP0
	bcf	R10K		;turn on voltage feed
	bcf	R10KA		;turn on other feed
	movlw	10101000b	;no prescale on TMR0
	movwf	OPTION
	movlw	0Eh		;ana0 on, left justify
	movwf	ADCON1
	bcf	RP0
	bsf	R10K		;10k pull up
	bcf	R10KA		;10k pull down
	movlw	81h		;read a/d on RA.0
	movwf	ADCON0
	clrf	dp		;no decimal
	movlw	0ffh
	movwf	TMR0		;set for single count overflow
		
:x1	btfsc	BUT1
	goto	:x2
	btfsc	BUT2
	goto	:x2
	goto	cycle
:x2	call	atod		;read the input
	movf	hi,w		;8 bit value
	sublw	40		;low enough for low?
	btfsc	c
	goto	:low
	movf	hi,w
	sublw	180
	btfss	c
	goto	:hi
:flo	movlw	40h		;'-'
	goto	:hl
:hi	movlw	76h		;'H'
	goto	:hl
:low	movlw	38h		;'L'
:hl	call	letter
	btfss	T0IF		;counter overflow?
	goto	:no
	bcf	T0IF
	movlw	0ffh
	movwf	TMR0		;set for next time
	clrf	timer
	movlw	73h		;'P'
	movwf	digits+1	;show pulse detected
:no	call	show1
	incf	timer
	movf	timer
	btfsc	z
	clrf	digits+1	;always flash off
	goto	:x1

; run display for a short time

show2	clrf	count
	goto	show:s2
show1	clrf	count
	bsf	count,7
	goto	show:s2
show	clrf	count
	bsf	count,6
:s2	call	segout		;show digits
	call	flash		;add dp
	movlw	10
	call	delay
	decfsz	count
	goto	:s2
	ret


; show one character and blank other 3

letter	movwf	digits
	goto	blnk2
blank	clrf	digits
blnk2	clrf	digits+1
	clrf	digits+2
	clrf	digits+3
	ret


; show 8 bit value on display

wtod	movwf	acc
	clrf	acc+1
	clrf	acc+2
	clrf	acc+3
	call	b2bcd
	call	format
	ret

; measure diode drops
; same as voltmeter but with 5v feed to probe

diode	bsf	RP0
	bcf	R10K		;enable feed resistor
	goto	vdx		;otherwise, like voltmeter

; voltmeter function, 0-5 volts

volt	bsf	RP0
vdx	movlw	8Eh		;enable ra.0 analog right justify
	movwf	ADCON1
	bcf	RP0
	bsf	R10K		;only used for 'diode' mode.
	movlw	81h		;select ana0 and enable a/d
	movwf	ADCON0

:v2	call	atod		;get 10 bits
	bcf	c
	rrf	hi
	rrf	low		;scale to 9 bits
	movf	low,w
	movwf	acc
	movf	hi,w
	movwf	acc+1
	clrf	acc+2
	clrf	acc+3
	call	b2bcd
	call	format
	bsf	digits+1,7
	movlw	200
	movwf	count
:v4	call	segout
	movlw	10
	call	delay
	decfsz	count
	goto	:v4
	btfsc	BUT1
	goto	:v2
	btfsc	BUT2
	goto	:v2
	goto	cycle

	
; read 16 bit a/d value in hi/low

atod	bsf	ADCON0,2
	nop
	nop
:chk	btfsc	ADCON0,2	;check for conversion complete
	goto	:chk
	bsf	RP0
	movf	ADRESL,w
	bcf	RP0
	movwf	low
	movf	ADRESH,w
	movwf	hi
	ret


; count events using TMR0 directly

dcount	bsf	RP0
	bsf	CLAMP		;disable clamp
	bcf	R10K		;use 10k only
	bsf	R150		;disconect other 10k
	bsf	R100K		;no 100k feed
	bsf	R20		;float direct resistor
	movlw	10101000b	;no prescale on TMR0
	movwf	OPTION
	movlw	6
	movwf	ADCON1		;all ra is digital
	bcf	RP0
	bsf	R10K		;use 10k pullup

:ic2	clrf	TMR0		;clear hardware timer
	clrf	xacc		;clear 32bit count
	clrf	xacc+1
	clrf	xacc+2
	clrf	xacc+3
:ic3	call	xfer
	call	b2bcd
	movlw	3
:ic5	call	xform		;show low 4 digits
	call	segout		;run display
	call	icup		;update counter
	btfss	BUT1		;button 1 ?
	goto	:ic9		;no - continue
	btfss	BUT2
	goto	:ic2		;reset counting.
	goto	:ic3		;continue count.
:ic9	btfss	BUT2		;both buttons?
	goto	cycle		;yes - exit.
	call	xfer
	call	b2bcd
	movlw	7
	goto	:ic5


; update 32bit count from hardware timer

icup	movf	TMR0,w		;hardware counter to accum
	movwf	xacc
	btfss	T0IF		;overflow
	ret			;not yet.
	bcf	T0IF
	incfsz	xacc+1
	ret
	incfsz	xacc+2
	ret
	incf	xacc+3
	ret


; move aux count to accum

xfer	movf	xacc,w
	movwf	acc
	movf	xacc+1,w
	movwf	acc+1
	movf	xacc+2,w
	movwf	acc+2
	movf	xacc+3,w
	movwf	acc+3
	ret
	

; measure frequency using TMR0 with 8-bit prescale
	
fcount	bsf	RP0
	bsf	CLAMP
	bsf	R10K		;no extra voltage
	bsf	R150		;disconect other 10k
	bcf	R100K		;use 100k pulldown
	bsf	R20		;float direct resistor
	movlw	10100111b	;256 count prescaler
	movwf	OPTION
	movlw	6
	movwf	ADCON1		;all ra is digital
	bcf	RP0
	bcf	R150		;pull down lightly
	clrf	acc
	clrf	acc+1
	clrf	acc+2
	clrf	acc+3
:f2	call	b2bcd
	call	format
	btfsc	BUT1
	goto	:f4
:f7	btfss	BUT2		;both buttons?
	goto	cycle
:fx	movlw	3
	call	xform
:f4	call	cnton
	call	onesec
	call	cntoff
	goto	:f2


; clear 32bit count and turn on TMR0

cnton	bcf	CLAMP
	clrf	TMR0		;clear timer and prescale
	clrf	ACC
	clrf	ACC+1
	clrf	ACC+2
	clrf	ACC+3
	bsf	RP0
	bsf	CLAMP		;tristate the clamp
	bcf	RP0
	ret
	
; turn off counter and pulse input to read out prescale
	
cntoff	bcf	CLAMP
	bsf	RP0
	bcf	CLAMP
	bcf	RP0
	movf	TMR0,w
	movwf	acc+1
	clrf	acc
:c2	decf	acc
	bsf	CLAMP
	bcf	CLAMP
	movf	TMR0,w
	xorwf	ACC+1,w
	btfsc	z
	goto	:c2
	ret


; add decimal point to display

flash	swapf	dp,w
	andlw	3		;bits 5,4 = digit nr.
	addlw	digits
	movwf	FSR
	btfsc	dp,7		;decimal used?
	bsf	0,7		;light decimal point
	incf	ftimer
	btfsc	z
	incf	ftimer+1
	btfss	ftimer+1,5
	goto	:no
	btfsc	dp,6		;flash mode?
	bcf	0,7		;flash back off
	ret
:no	nop
	ret

; open count gate for exactly 1.000000 second

onesec	movlw	LPTIME>
	movwf	timer
	movlw	LPTIME<
	movwf	timer+1
:x1	nop			;100 cycle loop count
	nop
:x2	call	fill		;time filler
	call	segout		;show last data
	call	flash		;include decimal
	call	dotime		;check for TMR0 overflow
	decfsz	timer
	goto	:x1
	decfsz	timer+1
	goto	:x2
	movlw	8
	movwf	timer
:x5	call	dotime
	decfsz	timer
	goto	:x5
	ret

; just kill some time

fill	movlw	16
	movwf	temp
:f2	decfsz	temp
	goto	:f2
	nop
	nop
	ret


; check for TMR0 rollover
; (constant execution times)

dotime	btfss	T0IF		;overflow?
	goto	:no
	bcf	T0IF		;clear the flag
	incf	acc+2		;increment next byte
	btfsc	z
	incf	acc+3		;and next on rollover
	ret
:no	nop
	nop
	nop
	ret
	

; squared delay function

delsq	movwf	dela+1
	movwf	dela+2
:d1	movf	dela+2,w
	movwf	dela
:d0	call	segout
	decfsz	dela
	goto	:d0
	decfsz	dela+1
	goto	:d1
	ret


; qubic delay function

delay	movwf	dela+2
:d2	movwf	dela+1
:d1	movwf	dela
:d0	decfsz	dela
	goto	:d0
	decfsz	dela+1
	goto	:d1
	decfsz	dela+2
	goto	:d2
	ret

; format used for capacitor measurements
; each count = 100pf

cform	call	first
	movf	count,w		;1st non zero
	call	dpcap
	movwf	dp
	andlw	0fh		;where to start
	movwf	count
	goto	format:f5


; *****	video pattern generation *****

vgen	bsf	RP0
	bcf	BLACK
	bcf	WHITE
	bcf	R100
	bcf	RP0
	bcf	R100
	bcf	rc,2
	bcf	rc,3
	movlw	0BFh
	movwf	rb
	
:vv2	call	vsync
	movlw	40
	movwf	count
:c1	nop
	call	bline
	decfsz	count
	goto	:c1
	movlw	15
	movwf	count	
:c2	call	dline
	call	dline4
	call	dline4
	call	bline4
	call	bline4
	call	bline4
	call	bline4
	call	bline4
	call	bline4
	call	bline4
	call	bline4
	call	bline4
	call	bline4
	nop
	decfsz	count
	goto	:c2
	movlw	17
	movwf	count
:c3	call	bline
	nop
	decfsz	count
	goto	:c3
	call	zline4
	btfsc	BUT2
	goto	:vv2
	goto	cycle

	
vsync	movlw	6
	movwf	count
:v2	bcf	BLACK
	call	dla10
	bsf	BLACK
	movlw	46
	call	vdela
	decfsz	count
	goto	:v2

	movlw	6
	movwf	count
:v3	bcf	BLACK
	movlw	46
	call	vdela
	bsf	BLACK
	call	dla8
	decfsz	count
	goto	:v3

	movlw	6
	movwf	count
:v4	bcf	BLACK
	call	dla10
	bsf	BLACK
	movlw	46
	call	vdela
	decfsz	count
	goto	:v4
	ret
	
zline4	call	dla4
zline	bcf	BLACK		;start h-sync
	call	dla16
	call	dla7
	bsf	BLACK
	call	vseg
	movlw	89
	call	vdela
	nop
	ret


bline4	call	dla4
bline	bcf	BLACK		;start h-sync
	call	dla16
	call	dla7
	bsf	BLACK
	movlw	93
	call	vdela
	nop
	ret

dline4	call	dla4
dline	bcf	BLACK		;start h-sync
	call	dla16
	call	dla7
	bsf	BLACK
	call	dla18
	call	dla18
	movlw	19
	movwf	temp
:d2	nop
	nop
	nop
	nop
	nop
	nop
	bsf	WHITE
	nop
	bcf	WHITE
	decfsz	temp
	goto	:d2
	movlw	5
	call	vdela
	ret


; delay in processor cycles (includes call/return)

dla18	nop
dla17	nop
dla16	nop
dla15	nop
dla14	nop
dla13	nop
dla12	nop
dla11	nop
dla10	nop
dla9	nop
dla8	nop
dla7	nop
dla6	nop
dla5	nop
dla4	ret

vdela	movwf	temp
:d2	decfsz	temp
	goto	:d2
	ret


; light alternate digits (12 ct)

vseg	incf	frame		;count frames
	bcf	rc,0
	bcf	rc,1
	btfss	frame,0
	bsf	rc,0
	btfsc	frame,0
	bsf	rc,1
	nop
	ret
	
	
; ****	end video gen *****


; turn off displays

clear	movlw	0F0h
	andwf	rc
	ret


; cycle thru individual segment drives on 4 digits
; (constant execution time: 30 instructions)
; ************** Do Not Modify *******************

segout	movlw	0F0h
	andwf	rc		;clear low 4 bits
	movf	dignr,w		;get digit nr
	addlw	digits		;base of segment list
	movwf	FSR
	movf	0,w		;get the data
	andwf	segmask,w	;mask the bit
	xorlw	0ffh		;invert
	movwf	rb		;one possible segment line low
	movf	dignr,w
	call	getano
	iorwf	rc		;set one high.
	bcf	c
	rlf	segmask
	btfsc	c
	goto	:nseg
	nop
	nop
	nop
	nop
	nop
	ret
:nseg	bsf	segmask,0	;rotate
	incf	dignr
	btfsc	dignr,2		;overflow?
	clrf	dignr
:bak	ret



; display digits from specified position

xform	movwf	count
	goto	format:f5

; format first 4 signifcant digits on display

format	call	first		;find 1st non-zero digit	
:fx	movf	count,w
	call	getdp		;get formating info
	movwf	dp
	andlw	0fh
	movwf	count
:f5	call	:f9		;get segment pattern
	movwf	digits
	call	:f9
	movwf	digits+1
	call	:f9
	movwf	digits+2
	call	:f9
	movwf	digits+3
	ret
			
:f9	movf	count,w
	decf	count
	call	getbd		;get bcd value
	call	digseg
	ret


; set count to 1st non zero digit

first	movlw	9		;start w/last digit
	movwf	count
:f2	movf	count,w		;get next digit
	call	getbd
	xorlw	0		;set z flag
	btfss	z
	ret			;hit non zero
	decfsz	count
	goto	:f2
	ret	


; get bcd digit specified by 'w'

getbd	movwf	temp		;save digit nr
	clrw
	btfsc	temp,7		;negative value?
	ret			;zero if negative.
	bcf	c
	rrf	temp		;find buffer offset
	movf	temp,w
	rlf	temp		;restore all bits
	addlw	bcd		;add start of bcd buff
	movwf	FSR		;set pointer
	movf	0,w		;get the byte
	movwf	data		;and store it.
	btfsc	temp,0		;low or hi nibble?
	swapf	data		;hi.
	movf	data,w
	andlw	0fh
	ret


; Convert 32-bit binary number at <acc> into a bcd number
; at <bcd>. Uses Mike Keitz's procedure for handling bcd 
; adjust; Modified Microchip AN526 for 32-bits.

b2bcd	movlw	32		; 32-bits
	movwf	ii		; make cycle counter
	clrf	bcd		; clear result area
	clrf	bcd+1
	clrf	bcd+2
	clrf	bcd+3
	clrf	bcd+4
	
b2bcd2	movlw	bcd		; make pointer
	movwf	FSR
	movlw	5
	movwf	cnt

; Mike's routine:

b2bcd3	movlw	33h		
	addwf	0 		; add to both nybbles
	btfsc	0,3		; test if low result > 7
	andlw	0f0h		; low result >7 so take the 3 out
	btfsc	0,7		; test if high result > 7
	andlw	0fh		; high result > 7 so ok
	subwf	0 		; any results <= 7, subtract back
	incf	FSR 		; point to next
	decfsz	cnt
	goto	b2bcd3
	
	rlf	acc+0 		; get another bit
	rlf	acc+1 
	rlf	acc+2 
	rlf	acc+3 
	rlf	bcd+0 		; put it into bcd
	rlf	bcd+1 
	rlf	bcd+2 
	rlf	bcd+3 
	rlf	bcd+4 
	decfsz	ii 		; all done?
	goto	b2bcd2		; no, loop
	ret


; read data 'w' at address hi/low

eeread	bcf	RP0
	bsf	RP1		;bank-2
	movf	low,w
	movwf	EEADR
	movf	hi,w
	movwf	EEADRH
	bsf	RP0		;bank-3
	bcf	EEPGD		;access data memory
	bsf	EERD		;start the read
	bcf	RP0		;bank-2
	movf	EEDATA,w
	bcf	RP1
	bcf	RP0
	ret


; write data 'w' at address hi/low

	
eewrite	bcf	RP0
	bsf	RP1		;bank-2
	movwf	EEDATA		;set data
	movf	low,w
	movwf	EEADR		;set address
	movf	hi,w
	movwf	EEADRH
	bsf	RP0		;bank-3
	bcf	EEPGD		;access data memory
	bsf	EEWREN		;start write operation
	movlw	55h
	movwf	EECON2
	movlw	0AAh
	movwf	EECON2
	bsf	EEWR
	nop
	nop
:e2	btfsc	EEWR		;wait for complete
	goto	:e2
	bcf	RP0
	bcf	RP1
	ret


s9600	call	clear
	bsf	RP0
	bsf	SOUT		;set to input
	bcf	RP0
	movf	baud,w
	movwf	temp
	bcf	c
	rlf	temp
	call	srq		;get low byte
	movwf	qtime
	call	srq
	movwf	qtime+1
	movlw	vbaud-vbase/4
	addwf	baud,w
	call	setmode

:s5	call	segout
	btfss	BUT2
	goto	:s8
	btfsc	BUT1
	goto	:s5
	btfss	BUT2
	goto	cycle
	call	setdir
	movlw	'A'
	movwf	sdata
	movlw	26
	movwf	scount

:s2	movf	sdata,w
	call	serout
	incf	sdata
	movlw	43
	call	delsq
	decfsz	scount
	goto	:s2

	movlw	13
	call	serout
	movlw	100
	call	delsq
	movlw	10
	call	serout
	movlw	220
	call	delsq
	btfss	BUT1		;still held down?
	goto	:s5		;repeat operation.
	goto	s9600

:s8	call	clear		;blank display
	movlw	20
	call	delay
	btfss	BUT1
	goto	cycle
	btfss	BUT2
	goto	:s8
	incf	baud
	bcf	baud,2		;limit 0-3
	bsf	SAVE
	goto	s9600


midi	movf	midic,w
	addlw	1		;convert to midi number
	call	wtod
	movlw	39h		;'C'
	movwf	digits
	movlw	76h		;'H'
	movwf	digits+1
	bsf	RP0
	bsf	SOUT		;set to input
	bcf	RP0
:s5	call	segout
	btfss	BUT2
	goto	:adj
	btfsc	BUT1
	goto	:s5
	call	setdir
	movlw	45		;31250 baud
	movwf	qtime
	clrf	qtime+1
	movf	midic,w		;get midi chan nr.
	iorlw	90h		;add note on command.
	call	serout		;note on
	movlw	60
	call	serout		;middle C
	movlw	40
	call	serout		;velocity
:s6	movlw	20
	call	delsq
	btfss	BUT1		;wait for button release
	goto	:s6
	movf	midic,w
	iorlw	90h
	call	serout
	movlw	60
	call	serout
	movlw	0
	call	serout		;send note off
	call	putopt		;maybe save channel
	movlw	250
	call	delsq		
	goto	midi
:adj	movlw	100
	call	delsq
	btfss	BUT1
	goto	cycle
	btfss	BUT2
	goto	:adj
	bsf	SAVE		;flag for saving new channel
	incf	midic		;next midi channel
	movlw	0fh
	andwf	midic		;roll over at 15
	goto	midi



; check for current state

setdir	bcf	DIR		;check for resting state
	btfss	SOUT
	bsf	DIR
	bsf	RP0
	bcf	SOUT		;set to output
	bcf	RP0
	ret


; send serial data from outdat

serout	clrf	count		;bit count
	bsf	count,3		;count=8
	movwf	data
	call	:zero		;start bit
:s2	call	:full
	btfsc	data,0
	call	:one
	btfss	data,0
	call	:zero
	rrf	data
	decfsz	count
	goto	:s2
	call	:full
	call	:one
	call	:full
	call	:full
	call	:full
	ret


:zero	btfss	DIR		;which way is up?
	bcf	SOUT
	btfsc	DIR
	bsf	SOUT
	ret

:one	btfss	DIR		;which way is up?
	bsf	SOUT
	btfsc	DIR
	bcf	SOUT
	ret


:full	movf	qtime+1,w
	movwf	temp+1
	movf	qtime,w
	movwf	temp
	incf	temp+1
	incf	temp
:f3	decfsz	temp
	goto	:f3
	decfsz	temp+1
	goto	:f3
	ret


	end
	
         